{########################################
 ##          -= SiDENGINE =-           ##
 ##                                    ##
 ##   Last Modify: 03/02/2021          ##
 ##   End Date:    __/__/____          ##
 ##                                    ##
 ##                                    ##
 ##  Copyright 2021 by OSG             ##
 ##                                    ##
 ########################################}

unit SiDEngine;

interface


//////////////////////// CHANGE PATH ONLY ////////////////////////

const
  EngDLL = 'Engine\SiDE.DLL';


//////////////////////// CHANGE PATH ONLY ////////////////////////

  {$I Keys.inc}
  {$I Cols.inc}

type
  TVec2D = record
    X, Y: Single;
  end;

type
  TVec3f = record
    X, Y, Z: Single;
  end;

type
  TSetRender = procedure();

  TSetUpdate = procedure();

/// SiDe CORE ///
type
  ISiDEng = interface
    function Init(Width, Height: integer; UWnd: LongWord; Fullscreen, Show_Cursor, ShowLogo: Boolean; TexFilter: byte): HRESULT; stdcall;
    procedure SetRender(Process: TSetRender); stdcall;
    procedure SetUpdate(Process: TSetUpdate); stdcall;
    procedure EngineLoop; stdcall;
    function GetDT(): Single; stdcall;
    function GetFPS(): Integer; stdcall;
    function GetWidth(): Integer; stdcall;
    function GetHeight(): Integer; stdcall;
    function GetWndDC: LongWord; stdcall;
    function GetWND: Cardinal; stdcall;
    function GetCoreTime(): Integer; stdcall;
    procedure ShowText(X, Y: Integer; Text: string; Color: Cardinal); stdcall;
    procedure UnLoad; stdcall;
  end;

procedure CreateEngine(out Engine: ISiDEng); stdcall; external EngDLL;
/// ////// ///



/// Render 2D ///

type
  IR2D = interface
    function LoadAtlas(FileName: string): Integer; stdcall;
    function LoadTexture(FileName: string; FWidth, FHeight: Integer): integer; stdcall;

    function LoadTextureEx(FileName: string; FWidth, FHeight: Integer; RT : Boolean): integer; stdcall;

    procedure UnloadTextures; stdcall;


    procedure LoadTextureRT(FWidth, FHeight: Integer); stdcall;
    procedure RenderRT(X, Y, W, H: Single; Alpha, Effect: byte; Color: Cardinal; Texture, Stage:Integer; Flags: Integer); stdcall;
    procedure RenderToTex(Targ,Clean: Boolean; ID:Integer); stdcall;
    procedure RenderToBackBuff; stdcall;
    procedure Render4(X, Y, Width, Height, Angle: Single; Texture, Flags: Integer); stdcall;
    procedure Render4A(X, Y, Width, Height, Angle, Frame: Single; Texture, Flags: Integer); stdcall;
    procedure Render4ExRegion(Name: string; X, Y, Width, Height, Angle: Single; Alpha: Byte; Color: Cardinal; Texture, Flags: Integer); stdcall;
    procedure Render4ExShRegion(Name: string; X, Y, Width, Height, Angle: Single; Shadows, Air: Boolean; ShadowX, ShadowY: Single; ShadowCapacity, Alpha: Byte; Color: Cardinal; Texture, Flags: Integer); stdcall;
    procedure Render4Ex(X, Y, Width, Height, Angle: Single; Alpha: Byte; Color: Cardinal; Texture, Flags: Integer); stdcall;
    procedure Render4AEx(X, Y, Width, Height, Angle, Frame: Single; Alpha: Byte; Color: Cardinal; Texture, Flags: Integer); stdcall;
    procedure Render4AExSh(X, Y, Width, Height, Angle{,Frame}: Single; Shadows, Air: Boolean; ShadowX, ShadowY: Single; ShadowCapacity, Alpha: Byte; Color: Cardinal; Texture, Flags: Integer); stdcall;
    procedure Render4ExUV(Name: string; X, Y, Width, Height, U1, V1, U2, V2, Angle: Single; Atlas, Shadow, InAir: Boolean; ShX, ShY: Single; ShCap, Alpha: Byte; Color: Cardinal; Frame: Single; Texture: Integer; Flags: Integer); stdcall;
    procedure Render4CA(X, Y, Width, Height, Angle: Single; A1, A2, A3, A4: Byte; C1, C2, C3, C4: Cardinal; Texture, Flags: Integer); stdcall;
    procedure RenderTiles(X, Y, Width, Height: Single; CX, CY: Integer; Angle: Single; Alpha: Byte; Color: Cardinal; Texture, Flags: Integer); stdcall;
    procedure DrawLine(X1, Y1, X2, Y2: Single; Color: Cardinal; Alpha: Byte); stdcall;
    procedure DrawRect(X, Y, Width, Height, Angle: Single; Color: Cardinal; Alpha: Byte); stdcall;
    procedure DrawText(X, Y: Single; _String: string; FontSize : Single; Texture, Xoff: Integer; Color: Cardinal; Flags: Byte);
    procedure DrawTextEx(X, Y: Single; _String: string; FontSize : Single; Texture, Xoff: Integer; C1, C2, C3, C4: Cardinal; Alpha: Byte; Angle: Single; Flags: Byte);
    procedure UpdateCamera(X, Y, Angle, Zoom: Single); stdcall;
    procedure SetEffect(Effect: byte); stdcall;
    function GetDips(): Integer; stdcall;
    function GetRndCount(): Integer; stdcall;
    function GetTexCount(): Integer; stdcall;
  end;

procedure CreateR2D(out ER2D: IR2D); stdcall; external EngDLL;
/// ////// ///


/// SOUND SYSTEM ///


/// ////// ///

/// INPUT SYSTEM ///

type
  IInput = interface
    function K_DOWNA(KCode: byte): boolean; stdcall;
    function M_X: Integer; stdcall;
    function M_Y: Integer; stdcall;
    function M_Move: boolean; stdcall;
    procedure M_Hold; stdcall;
    function M_Click(button: byte): boolean; stdcall;
    function M_DClick(button: byte): boolean; stdcall;
    function M_Down(button: byte): boolean; stdcall;
    function M_Press(button: byte): boolean; stdcall;
    function M_WheelUp: boolean; stdcall;
    function M_WheelDown: boolean; stdcall;
  end;

procedure CreateInput(out EInput: IInput); stdcall; external EngDLL;

/// ////// ///


/// OTHER STUFF ///

function StrToIntS(Str: string): Integer; stdcall;

function IntToStr(Val: Integer): PChar; stdcall;

function FloatToStr(Val: Single): PChar; stdcall;


/// ////// ///

/// MOUSE STUFF ///
const
  M_RightB = 0;
  M_LeftB = 1;
  M_MidB = 2;


/// ////// ///

function InRect(MX, MY, MW, MH, X, Y, W, H: Integer): boolean;

/// ////// ///


/// CAMERA ///

type
  T2DCamera = record
    X, Y, Angle, Zoom: Single;
    Active: Boolean;
  end;
/// ////// ///

var
  Eng: ISiDEng;
  R2D: IR2D;
  Input: IInput;
  Camera: T2DCamera;
  FStr: string;

implementation

function StrToIntS;
var
  Num: Integer;
begin
  Val(Str, Result, Num);
end;

function IntToStr;
begin
  Str(Val, FStr);
  Result := PChar(FStr);
end;

function FloatToStr;
begin
  Str(Val: 3: 7, FStr);
  Result := PChar(FStr);
end;

function InRect(MX, MY, MW, MH, X, Y, W, H: Integer): boolean;
begin

  if (MX + MW > X) and (MX < X + W) and (MY + MH > Y) and (MY < Y + H) then
    Result := True
  else
    Result := False;
end;

initialization
  Camera.Active := true;

end.
